/*
 * Jicofo, the Jitsi Conference Focus.
 *
 * Copyright @ 2015 Atlassian Pty Ltd
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jitsi.impl.protocol.xmpp.colibri;

import org.jitsi.xmpp.extensions.colibri.*;
import org.jitsi.xmpp.extensions.jingle.*;

import org.jitsi.eventadmin.*;
import org.jitsi.protocol.xmpp.*;
import org.jitsi.protocol.xmpp.colibri.*;

import org.jitsi.utils.logging.*;
import org.jivesoftware.smack.provider.*;

/**
 * Default implementation of {@link OperationSetColibriConference} that uses
 * Smack for handling XMPP connection. Handles conference state, allocates and
 * expires channels.
 *
 * @author Pawel Domas
 */
public class OperationSetColibriConferenceImpl
    implements OperationSetColibriConference
{
    private final static Logger logger
            = Logger.getLogger(OperationSetColibriConferenceImpl.class);

    private XmppConnection connection;

    private EventAdmin eventAdmin;

    /**
     * Initializes this operation set.
     *
     * @param connection Smack XMPP connection impl that will be used to send
     *                   and receive XMPP packets.
     * @param eventAdmin the <tt>EventAdmin</tt> which will be used by
     *                   the underlying components to emit events.
     */
    public void initialize(XmppConnection connection, EventAdmin eventAdmin)
    {
        this.connection = connection;
        this.eventAdmin = eventAdmin;

        // Register Colibri
        ProviderManager.addIQProvider(
            ColibriConferenceIQ.ELEMENT_NAME,
            ColibriConferenceIQ.NAMESPACE,
            new ColibriIQProvider());

        // register Jingle
        ProviderManager.addIQProvider(
            JingleIQ.ELEMENT_NAME,
            JingleIQ.NAMESPACE,
            new JingleIQProvider());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public ColibriConference createNewConference()
    {
        ColibriConference conf
            = new ColibriConferenceImpl(connection, eventAdmin);
        logger.info("Conference created: " + conf);
        return conf;
    }
}
